/*    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2013-
 *    Salvatore Ruggieri
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package rst.table;

import java.util.ArrayList;
import java.util.Collections;
import java.util.TreeMap;

/** Column of indexes to elements of Double type. */
public class DoubleColumn extends TypedColumn<Double> {

	/** Constructor. */
	public DoubleColumn(String name, int size)  {
		super(name, size);		
	}
	
	public void setString(int pos, String value) {
		Double d = Double.parseDouble(value);
		set(pos, d);
	}
	
	/** Re-order value codes ascending. */
	public void sortCodes() {
		ArrayList<Double> decode2 = new ArrayList<Double>(decode);
		Collections.sort(decode2);
		encode = new TreeMap<Double, Integer>();
		int size = decode2.size();
		for(int i=0; i<size; ++i ) {
			Double d = decode2.get(i);
			encode.put(d, i);
		}
		int sz = index.length;
		for(int i=0; i<sz; ++i)
			index[i] = encode.get(decode.get(index[i]));
		decode = decode2;
	}

	public int getIdx(String value) {
		Integer ret = encode.get(Double.parseDouble(value));
		if( ret == null)
			throw new RuntimeException("Column::getIdx - unknown value " + value + " on column " + name);
		return ret;	
	}
	
	public double distanceEuclidian(int pos1, int pos2) {
		double d = decode.get(index[pos1]) - decode.get(index[pos2]);
		return d*d;
	}

	public boolean isNumeric() {
		return true;
	}
}
