/*    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2013-
 *    Salvatore Ruggieri
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package rst.util.data;

import java.io.Serializable;
import java.util.TreeSet;

/** Cache of T objects, each one with a measure value. */
public class Cache<T> implements Serializable {
	/** Serial ID. */
	private static final long serialVersionUID = -4784858059344698001L;
	/** Max cache size. */
	public int maxsize;
	/** Max measure element in cache. */
	private double maxmeasure;
	/** Cache. */
	public TreeSet<Pair<Double,T>> cache;
	/** Constructor. */
	public Cache(int maxsize) {
		this.maxsize = maxsize;
		cache = new TreeSet<Pair<Double,T>>();
		maxmeasure = Double.MAX_VALUE;
	}
	/** Add in cache. */
	public void add(T val, double measure) {
		if(measure > maxmeasure )
			return;
		cache.add( new Pair<Double,T>(measure, val) );
		if(cache.size() > maxsize) {
			Pair<Double,T> last = cache.pollLast();
			maxmeasure = last.first;
		}
	}
	/** String representation. */
	public String toString()
	{
		String res = "";
		for(Pair<Double,T> v : cache)
			res += "(" + v.first + ", " + v.second + ") ";
		return res;
	}
	/** Test method. */
	public static void main(String[] args) {
		final int sz = 3;
		Cache<String> cache = new Cache<String>(sz);
		cache.add("a", 2);
		cache.add("b", 1);
		cache.add("c", 3);
		System.out.println(cache);
		cache.add("d", 2);
		System.out.println(cache);
	}
}
